@@ -45,11 +45,6 @@ Blacklist::Blacklist() {
 }
 
 Blacklist::~Blacklist() {
-  // Free the map keys
-  BlacklistMap::iterator i;
-  for (i=blm.begin(); i!=blm.end(); i++) {
-    strFree((char*)(*i).first);
-  }
 }
 
 bool Blacklist::isBlackmarked(const char* name) {
@@ -65,7 +60,7 @@ bool Blacklist::isBlackmarked(const char* name) {
     bi.marks = 1;
     bi.blockUntil = 0;
     bi.blockTimeout = initialTimeout;
-    blm[strDup(name)] = bi;
+    blm[name] = bi;
     i = blm.find(name);
   }
 
@@ -92,9 +87,5 @@ bool Blacklist::isBlackmarked(const char* name) {
 }
 
 void Blacklist::clearBlackmark(const char* name) {
-  BlacklistMap::iterator i = blm.find(name);
-  if (i != blm.end()) {
-    strFree((char*)(*i).first);
-    blm.erase(i);
-  }
+  blm.erase(name);
 }
