@@ -32,6 +32,7 @@
 #include <rfb/CMsgWriter.h>
 #include <rfb/CSecurity.h>
 #include <rfb/Decoder.h>
+#include <rfb/KeysymStr.h>
 #include <rfb/Security.h>
 #include <rfb/SecurityClient.h>
 #include <rfb/CConnection.h>
@@ -52,14 +53,14 @@ CConnection::CConnection()
     supportsDesktopResize(false), supportsLEDState(false),
     is(0), os(0), reader_(0), writer_(0),
     shared(false),
-    state_(RFBSTATE_UNINITIALISED), serverName(strDup("")),
+    state_(RFBSTATE_UNINITIALISED),
     pendingPFChange(false), preferredEncoding(encodingTight),
     compressLevel(2), qualityLevel(-1),
     formatChange(false), encodingChange(false),
     firstUpdate(true), pendingUpdate(false), continuousUpdates(false),
     forceNonincremental(true),
     framebuffer(NULL), decoder(this),
-    serverClipboard(NULL), hasLocalClipboard(false)
+    hasRemoteClipboard(false), hasLocalClipboard(false)
 {
 }
 
@@ -72,7 +73,7 @@ void CConnection::setServerName(const char* name_)
 {
   if (name_ == NULL)
     name_ = "";
-  serverName.replaceBuf(strDup(name_));
+  serverName = name_;
 }
 
 void CConnection::setStreams(rdr::InStream* is_, rdr::OutStream* os_)
@@ -93,10 +94,10 @@ void CConnection::setFramebuffer(ModifiablePixelBuffer* fb)
   if ((framebuffer != NULL) && (fb != NULL)) {
     Rect rect;
 
-    const rdr::U8* data;
+    const uint8_t* data;
     int stride;
 
-    const rdr::U8 black[4] = { 0, 0, 0, 0 };
+    const uint8_t black[4] = { 0, 0, 0, 0 };
 
     // Copy still valid area
 
@@ -163,7 +164,7 @@ bool CConnection::processVersionMsg()
   if (!is->hasData(12))
     return false;
 
-  is->readBytes(verStr, 12);
+  is->readBytes((uint8_t*)verStr, 12);
   verStr[12] = '\0';
 
   if (sscanf(verStr, "RFB %03d.%03d\n",
@@ -192,7 +193,7 @@ bool CConnection::processVersionMsg()
 
   sprintf(verStr, "RFB %03d.%03d\n",
           server.majorVersion, server.minorVersion);
-  os->writeBytes(verStr, 12);
+  os->writeBytes((const uint8_t*)verStr, 12);
   os->flush();
 
   state_ = RFBSTATE_SECURITY_TYPES;
@@ -210,7 +211,7 @@ bool CConnection::processSecurityTypesMsg()
 
   int secType = secTypeInvalid;
 
-  std::list<rdr::U8> secTypes;
+  std::list<uint8_t> secTypes;
   secTypes = security.GetEnabledSecTypes();
 
   if (server.isVersion(3,3)) {
@@ -225,7 +226,7 @@ bool CConnection::processSecurityTypesMsg()
       state_ = RFBSTATE_SECURITY_REASON;
       return true;
     } else if (secType == secTypeNone || secType == secTypeVncAuth) {
-      std::list<rdr::U8>::iterator i;
+      std::list<uint8_t>::iterator i;
       for (i = secTypes.begin(); i != secTypes.end(); i++)
         if (*i == secType) {
           secType = *i;
@@ -259,10 +260,10 @@ bool CConnection::processSecurityTypesMsg()
       return true;
     }
 
-    std::list<rdr::U8>::iterator j;
+    std::list<uint8_t>::iterator j;
 
     for (int i = 0; i < nServerSecTypes; i++) {
-      rdr::U8 serverSecType = is->readU8();
+      uint8_t serverSecType = is->readU8();
       vlog.debug("Server offers security type %s(%d)",
                  secTypeName(serverSecType), serverSecType);
 
@@ -355,17 +356,17 @@ bool CConnection::processSecurityReasonMsg()
 
   is->setRestorePoint();
 
-  rdr::U32 len = is->readU32();
+  uint32_t len = is->readU32();
   if (!is->hasDataOrRestore(len))
     return false;
   is->clearRestorePoint();
 
-  CharArray reason(len + 1);
-  is->readBytes(reason.buf, len);
-  reason.buf[len] = '\0';
+  std::vector<char> reason(len + 1);
+  is->readBytes((uint8_t*)reason.data(), len);
+  reason[len] = '\0';
 
   state_ = RFBSTATE_INVALID;
-  throw AuthFailureException(reason.buf);
+  throw AuthFailureException(reason.data());
 }
 
 bool CConnection::processInitMsg()
@@ -405,8 +406,6 @@ void CConnection::close()
   reader_ = NULL;
   delete writer_;
   writer_ = NULL;
-  strFree(serverClipboard);
-  serverClipboard = NULL;
 }
 
 void CConnection::setDesktopSize(int w, int h)
@@ -545,18 +544,16 @@ void CConnection::serverCutText(const char* str)
 {
   hasLocalClipboard = false;
 
-  strFree(serverClipboard);
-  serverClipboard = NULL;
-
-  serverClipboard = latin1ToUTF8(str);
+  serverClipboard = str;
+  hasRemoteClipboard = true;
 
   handleClipboardAnnounce(true);
 }
 
-void CConnection::handleClipboardCaps(rdr::U32 flags,
-                                      const rdr::U32* lengths)
+void CConnection::handleClipboardCaps(uint32_t flags,
+                                      const uint32_t* lengths)
 {
-  rdr::U32 sizes[] = { 0 };
+  uint32_t sizes[] = { 0 };
 
   CMsgHandler::handleClipboardCaps(flags, lengths);
 
@@ -568,7 +565,7 @@ void CConnection::handleClipboardCaps(rdr::U32 flags,
                                sizes);
 }
 
-void CConnection::handleClipboardRequest(rdr::U32 flags)
+void CConnection::handleClipboardRequest(uint32_t flags)
 {
   if (!(flags & rfb::clipboardUTF8)) {
     vlog.debug("Ignoring clipboard request for unsupported formats 0x%x", flags);
@@ -581,16 +578,15 @@ void CConnection::handleClipboardRequest(rdr::U32 flags)
   handleClipboardRequest();
 }
 
-void CConnection::handleClipboardPeek(rdr::U32 flags)
+void CConnection::handleClipboardPeek()
 {
   if (server.clipboardFlags() & rfb::clipboardNotify)
     writer()->writeClipboardNotify(hasLocalClipboard ? rfb::clipboardUTF8 : 0);
 }
 
-void CConnection::handleClipboardNotify(rdr::U32 flags)
+void CConnection::handleClipboardNotify(uint32_t flags)
 {
-  strFree(serverClipboard);
-  serverClipboard = NULL;
+  hasRemoteClipboard = false;
 
   if (flags & rfb::clipboardUTF8) {
     hasLocalClipboard = false;
@@ -600,22 +596,25 @@ void CConnection::handleClipboardNotify(rdr::U32 flags)
   }
 }
 
-void CConnection::handleClipboardProvide(rdr::U32 flags,
+void CConnection::handleClipboardProvide(uint32_t flags,
                                          const size_t* lengths,
-                                         const rdr::U8* const* data)
+                                         const uint8_t* const* data)
 {
   if (!(flags & rfb::clipboardUTF8)) {
     vlog.debug("Ignoring clipboard provide with unsupported formats 0x%x", flags);
     return;
   }
 
-  strFree(serverClipboard);
-  serverClipboard = NULL;
-
+  // FIXME: This conversion magic should be in CMsgReader
+  if (!isValidUTF8((const char*)data[0], lengths[0])) {
+    vlog.error("Invalid UTF-8 sequence in clipboard - ignoring");
+    return;
+  }
   serverClipboard = convertLF((const char*)data[0], lengths[0]);
+  hasRemoteClipboard = true;
 
   // FIXME: Should probably verify that this data was actually requested
-  handleClipboardData(serverClipboard);
+  handleClipboardData(serverClipboard.c_str());
 }
 
 void CConnection::authSuccess()
@@ -635,18 +634,18 @@ void CConnection::handleClipboardRequest()
 {
 }
 
-void CConnection::handleClipboardAnnounce(bool available)
+void CConnection::handleClipboardAnnounce(bool /*available*/)
 {
 }
 
-void CConnection::handleClipboardData(const char* data)
+void CConnection::handleClipboardData(const char* /*data*/)
 {
 }
 
 void CConnection::requestClipboard()
 {
-  if (serverClipboard != NULL) {
-    handleClipboardData(serverClipboard);
+  if (hasRemoteClipboard) {
+    handleClipboardData(serverClipboard.c_str());
     return;
   }
 
@@ -681,9 +680,10 @@ void CConnection::announceClipboard(bool available)
 void CConnection::sendClipboardData(const char* data)
 {
   if (server.clipboardFlags() & rfb::clipboardProvide) {
-    CharArray filtered(convertCRLF(data));
-    size_t sizes[1] = { strlen(filtered.buf) + 1 };
-    const rdr::U8* data[1] = { (const rdr::U8*)filtered.buf };
+    // FIXME: This conversion magic should be in CMsgWriter
+    std::string filtered(convertCRLF(data));
+    size_t sizes[1] = { filtered.size() + 1 };
+    const uint8_t* data[1] = { (const uint8_t*)filtered.c_str() };
 
     if (unsolicitedClipboardAttempt) {
       unsolicitedClipboardAttempt = false;
@@ -697,9 +697,7 @@ void CConnection::sendClipboardData(const char* data)
 
     writer()->writeClipboardProvide(rfb::clipboardUTF8, sizes, data);
   } else {
-    CharArray latin1(utf8ToLatin1(data));
-
-    writer()->writeClientCutText(latin1.buf);
+    writer()->writeClientCutText(data);
   }
 }
 
@@ -747,14 +745,14 @@ void CConnection::setQualityLevel(int level)
 
 void CConnection::setPF(const PixelFormat& pf)
 {
-  if (server.pf().equal(pf) && !formatChange)
+  if (server.pf() == pf && !formatChange)
     return;
 
   nextPF = pf;
   formatChange = true;
 }
 
-void CConnection::fence(rdr::U32 flags, unsigned len, const char data[])
+void CConnection::fence(uint32_t flags, unsigned len, const uint8_t data[])
 {
   CMsgHandler::fence(flags, len, data);
 
@@ -819,7 +817,7 @@ void CConnection::requestNewUpdate()
 
 void CConnection::updateEncodings()
 {
-  std::list<rdr::U32> encodings;
+  std::list<uint32_t> encodings;
 
   if (supportsLocalCursor) {
     encodings.push_back(pseudoEncodingCursorWithAlpha);
