@@ -1,4 +1,5 @@
 /* Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
+ * Copyright 2014-2022 Pierre Ossman for Cendio AB
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,14 +23,24 @@
 
 namespace rfb {
 
+  class PixelFormat;
+
   class HextileDecoder : public Decoder {
   public:
-    static Decoder* create(CMsgReader* reader);
-    virtual void readRect(const Rect& r, CMsgHandler* handler);
+    HextileDecoder();
     virtual ~HextileDecoder();
+    virtual bool readRect(const Rect& r, rdr::InStream* is,
+                          const ServerParams& server, rdr::OutStream* os);
+    virtual void decodeRect(const Rect& r, const uint8_t* buffer,
+                            size_t buflen, const ServerParams& server,
+                            ModifiablePixelBuffer* pb);
   private:
-    HextileDecoder(CMsgReader* reader);
-    CMsgReader* reader;
+    template<class T>
+    inline T readPixel(rdr::InStream* is);
+    template<class T>
+    void hextileDecode(const Rect& r, rdr::InStream* is,
+                       const PixelFormat& pf,
+                       ModifiablePixelBuffer* pb);
   };
 }
 #endif
