@@ -23,21 +23,20 @@
 #ifndef __RFB_INPUTHANDLER_H__
 #define __RFB_INPUTHANDLER_H__
 
-#include <rdr/types.h>
+#include <stdint.h>
+
 #include <rfb/Rect.h>
-#include <rfb/util.h>
 
 namespace rfb {
 
   class InputHandler {
   public:
     virtual ~InputHandler() {}
-    virtual void keyEvent(rdr::U32 __unused_attr keysym,
-                          rdr::U32 __unused_attr keycode,
-                          bool __unused_attr down) { }
-    virtual void pointerEvent(const Point& __unused_attr pos,
-                              int __unused_attr buttonMask) { }
-    virtual void clientCutText(const char* __unused_attr str) { }
+    virtual void keyEvent(uint32_t /*keysym*/, uint32_t /*keycode*/,
+                          bool /*down*/) { }
+    virtual void pointerEvent(const Point& /*pos*/,
+                              int /*buttonMask*/) { }
+    virtual void clientCutText(const char* /*str*/) { }
   };
 
 }
