@@ -18,13 +18,16 @@
 
 // -=- Logger.cxx - support for the Logger and LogWriter classes
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 
 #include <rfb/Logger.h>
 #include <rfb/LogWriter.h>
-#include <rfb/util.h>
 
 using namespace rfb;
 
