@@ -1,4 +1,5 @@
 /* Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
+ * Copyright 2014-2022 Pierre Ossman for Cendio AB
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,15 +23,23 @@
 
 namespace rfb {
 
+  class PixelFormat;
+
   class RREDecoder : public Decoder {
   public:
     RREDecoder();
     virtual ~RREDecoder();
     virtual bool readRect(const Rect& r, rdr::InStream* is,
                           const ServerParams& server, rdr::OutStream* os);
-    virtual void decodeRect(const Rect& r, const void* buffer,
+    virtual void decodeRect(const Rect& r, const uint8_t* buffer,
                             size_t buflen, const ServerParams& server,
                             ModifiablePixelBuffer* pb);
+  private:
+    template<class T>
+    inline T readPixel(rdr::InStream* is);
+    template<class T>
+    void rreDecode(const Rect& r, rdr::InStream* is,
+                   const PixelFormat& pf, ModifiablePixelBuffer* pb);
   };
 }
 #endif
