@@ -16,6 +16,11 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  * USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <rdr/OutStream.h>
 #include <rfb/encodings.h>
 #include <rfb/SConnection.h>
@@ -39,9 +44,10 @@ bool RawEncoder::isSupported()
   return true;
 }
 
-void RawEncoder::writeRect(const PixelBuffer* pb, const Palette& palette)
+void RawEncoder::writeRect(const PixelBuffer* pb,
+                           const Palette& /*palette*/)
 {
-  const rdr::U8* buffer;
+  const uint8_t* buffer;
   int stride;
 
   rdr::OutStream* os;
@@ -62,7 +68,7 @@ void RawEncoder::writeRect(const PixelBuffer* pb, const Palette& palette)
 
 void RawEncoder::writeSolidRect(int width, int height,
                                 const PixelFormat& pf,
-                                const rdr::U8* colour)
+                                const uint8_t* colour)
 {
   rdr::OutStream* os;
   int pixels, pixel_size;
