@@ -17,16 +17,20 @@
  */
 #ifndef __RFB_USERPASSWDGETTER_H__
 #define __RFB_USERPASSWDGETTER_H__
+
+#include <string>
+
 namespace rfb {
   class UserPasswdGetter {
   public:
     // getUserPasswd gets the username and password.  This might involve a
     // dialog, getpass(), etc.  The user buffer pointer can be null, in which
-    // case no user name will be retrieved.  The caller MUST delete [] the
-    // result(s).
-    virtual void getUserPasswd(bool secure, char** user, char** password)=0;
+    // case no user name will be retrieved.
+    virtual void getUserPasswd(bool secure, std::string* user,
+                               std::string* password)=0;
 
     virtual ~UserPasswdGetter() {}
   };
 }
+
 #endif
