@@ -59,7 +59,7 @@ private:
   virtual void overrun(size_t needed);
 
   int offset;
-  rdr::U8 buf[131072];
+  uint8_t buf[131072];
 };
 
 class CConn : public rfb::CConnection {
@@ -69,11 +69,11 @@ public:
 
   virtual void initDone();
   virtual void setPixelFormat(const rfb::PixelFormat& pf);
-  virtual void setCursor(int, int, const rfb::Point&, const rdr::U8*);
+  virtual void setCursor(int, int, const rfb::Point&, const uint8_t*);
   virtual void setCursorPos(const rfb::Point&);
   virtual void framebufferUpdateStart();
   virtual void framebufferUpdateEnd();
-  virtual void setColourMapEntries(int, int, rdr::U16*);
+  virtual void setColourMapEntries(int, int, uint16_t*);
   virtual void bell();
   virtual void serverCutText(const char*);
 
@@ -139,13 +139,13 @@ void CConn::initDone()
                                              server.height()));
 }
 
-void CConn::setPixelFormat(const rfb::PixelFormat& pf)
+void CConn::setPixelFormat(const rfb::PixelFormat& /*pf*/)
 {
   // Override format
   CConnection::setPixelFormat(filePF);
 }
 
-void CConn::setCursor(int, int, const rfb::Point&, const rdr::U8*)
+void CConn::setCursor(int, int, const rfb::Point&, const uint8_t*)
 {
 }
 
@@ -169,7 +169,7 @@ void CConn::framebufferUpdateEnd()
   cpuTime += getCpuCounter();
 }
 
-void CConn::setColourMapEntries(int, int, rdr::U16*)
+void CConn::setColourMapEntries(int, int, uint16_t*)
 {
 }
 
