@@ -26,6 +26,10 @@
  * are not encoded in the file and must be specified by the user.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #define __USE_MINGW_ANSI_STDIO 1
 
 #include <stdio.h>
@@ -37,6 +41,8 @@
 #include <rdr/OutStream.h>
 #include <rdr/FileInStream.h>
 
+#include <rfb/AccessRights.h>
+
 #include <rfb/PixelFormat.h>
 
 #include <rfb/CConnection.h>
@@ -64,7 +70,7 @@ static rfb::BoolParameter translate("translate",
 static const rfb::PixelFormat fbPF(32, 24, false, true, 255, 255, 255, 0, 8, 16);
 
 // Encodings to use
-static const rdr::S32 encodings[] = {
+static const int32_t encodings[] = {
   rfb::encodingTight, rfb::encodingCopyRect, rfb::encodingRRE,
   rfb::encodingHextile, rfb::encodingZRLE, rfb::pseudoEncodingLastRect,
   rfb::pseudoEncodingQualityLevel0 + 8,
@@ -81,7 +87,7 @@ private:
   virtual void overrun(size_t needed);
 
   int offset;
-  rdr::U8 buf[131072];
+  uint8_t buf[131072];
 };
 
 class CConn : public rfb::CConnection {
@@ -94,12 +100,12 @@ public:
 
   virtual void initDone() {};
   virtual void resizeFramebuffer();
-  virtual void setCursor(int, int, const rfb::Point&, const rdr::U8*);
+  virtual void setCursor(int, int, const rfb::Point&, const uint8_t*);
   virtual void setCursorPos(const rfb::Point&);
   virtual void framebufferUpdateStart();
   virtual void framebufferUpdateEnd();
   virtual bool dataRect(const rfb::Rect&, int);
-  virtual void setColourMapEntries(int, int, rdr::U16*);
+  virtual void setColourMapEntries(int, int, uint16_t*);
   virtual void bell();
   virtual void serverCutText(const char*);
 
@@ -130,7 +136,7 @@ public:
 
   void getStats(double&, unsigned long long&, unsigned long long&);
 
-  virtual void setAccessRights(AccessRights ar);
+  virtual void setAccessRights(rfb::AccessRights ar);
 
   virtual void setDesktopSize(int fb_width, int fb_height,
                               const rfb::ScreenSet& layout);
@@ -213,7 +219,7 @@ void CConn::resizeFramebuffer()
   setFramebuffer(pb);
 }
 
-void CConn::setCursor(int, int, const rfb::Point&, const rdr::U8*)
+void CConn::setCursor(int, int, const rfb::Point&, const uint8_t*)
 {
 }
 
@@ -261,7 +267,7 @@ bool CConn::dataRect(const rfb::Rect &r, int encoding)
   return true;
 }
 
-void CConn::setColourMapEntries(int, int, rdr::U16*)
+void CConn::setColourMapEntries(int, int, uint16_t*)
 {
 }
 
@@ -299,6 +305,7 @@ void Manager::getStats(double& ratio, unsigned long long& encodedBytes,
 }
 
 SConn::SConn()
+: SConnection(rfb::AccessDefault)
 {
   out = new DummyOutStream;
   setStreams(NULL, out);
@@ -325,12 +332,11 @@ void SConn::getStats(double& ratio, unsigned long long& bytes,
   manager->getStats(ratio, bytes, rawEquivalent);
 }
 
-void SConn::setAccessRights(AccessRights ar)
+void SConn::setAccessRights(rfb::AccessRights)
 {
 }
 
-void SConn::setDesktopSize(int fb_width, int fb_height,
-                           const rfb::ScreenSet& layout)
+void SConn::setDesktopSize(int, int, const rfb::ScreenSet&)
 {
 }
 
