@@ -93,7 +93,7 @@ TestWindow::~TestWindow()
 
 void TestWindow::start(int width, int height)
 {
-  rdr::U32 pixel;
+  uint32_t pixel;
 
   stop();
 
@@ -176,7 +176,7 @@ void TestWindow::update()
 
 void TestWindow::changefb()
 {
-  rdr::U32 pixel;
+  uint32_t pixel;
 
   pixel = rand();
   fb->fillRect(fb->getRect(), &pixel);
@@ -195,7 +195,7 @@ void TestWindow::timer(void* data)
 void PartialTestWindow::changefb()
 {
   rfb::Rect r;
-  rdr::U32 pixel;
+  uint32_t pixel;
 
   r = fb->getRect();
   r.tl.x += w() / 4;
@@ -322,7 +322,6 @@ static void dotest(TestWindow* win)
   double time[3];
 
   double delay, rate;
-  char s[1024];
 
   // Run the test several times at different resolutions...
   dosubtest(win, 800, 600, &pixels[0], &frames[0], &time[0]);
@@ -369,16 +368,14 @@ static void dotest(TestWindow* win)
   }
 
   fprintf(stderr, "Rendering delay: %g ms/frame\n", delay * 1000.0);
-  if (rate == 0.0)
-    strcpy(s, "N/A pixels/s");
-  else
-    rfb::siPrefix(1.0 / rate, "pixels/s", s, sizeof(s));
-  fprintf(stderr, "Rendering rate: %s\n", s);
+  fprintf(stderr, "Rendering rate: %s\n",
+          (rate == 0.0) ? "N/A pixels/s" :
+                          rfb::siPrefix(1.0 / rate, "pixels/s").c_str());
   fprintf(stderr, "Maximum FPS: %g fps @ 1920x1080\n",
           1.0 / (delay + rate * 1920 * 1080));
 }
 
-int main(int argc, char** argv)
+int main(int /*argc*/, char** /*argv*/)
 {
   TestWindow* win;
 
