@@ -1233,7 +1233,7 @@ void testIgnore()
   testIgnoreAfterGesture();
 }
 
-int main(int argc, char** argv)
+int main(int /*argc*/, char** /*argv*/)
 {
   testOneTap();
   testTwoTap();
