@@ -35,6 +35,12 @@
 class Geometry;
 class XPixelBuffer;
 
+struct AddedKeySym
+{
+  KeySym keysym;
+  KeyCode keycode;
+};
+
 // number of XKb indicator leds to handle
 #define XDESKTOP_N_LEDS 3
 
@@ -47,18 +53,15 @@ public:
   virtual ~XDesktop();
   void poll();
   // -=- SDesktop interface
-  virtual void start(rfb::VNCServer* vs);
+  virtual void init(rfb::VNCServer* vs);
+  virtual void start();
   virtual void stop();
   virtual void terminate();
   bool isRunning();
   virtual void queryConnection(network::Socket* sock,
                                const char* userName);
   virtual void pointerEvent(const rfb::Point& pos, int buttonMask);
-  KeyCode XkbKeysymToKeycode(Display* dpy, KeySym keysym);
-  KeyCode addKeysym(Display* dpy, KeySym keysym);
-  void deleteAddedKeysyms(Display* dpy);
-  KeyCode keysymToKeycode(Display* dpy, KeySym keysym);
-  virtual void keyEvent(rdr::U32 keysym, rdr::U32 xtcode, bool down);
+  virtual void keyEvent(uint32_t keysym, uint32_t xtcode, bool down);
   virtual void clientCutText(const char* str);
   virtual unsigned int setScreenLayout(int fb_width, int fb_height,
                                        const rfb::ScreenSet& layout);
@@ -81,7 +84,7 @@ protected:
   bool haveXtest;
   bool haveDamage;
   int maxButtons;
-  std::map<KeySym, KeyCode> addedKeysyms;
+  std::list<AddedKeySym> addedKeysyms;
   std::map<KeySym, KeyCode> pressedKeys;
   bool running;
 #ifdef HAVE_XDAMAGE
@@ -101,7 +104,18 @@ protected:
   unsigned ledState;
   const unsigned short *codeMap;
   unsigned codeMapLen;
+
+protected:
+#ifdef HAVE_XTEST
+  KeyCode XkbKeysymToKeycode(Display* dpy, KeySym keysym);
+  KeyCode getReusableKeycode(XkbDescPtr xkb);
+  KeyCode addKeysym(Display* dpy, KeySym keysym);
+  void deleteAddedKeysyms(Display* dpy);
+  KeyCode keysymToKeycode(Display* dpy, KeySym keysym);
+#endif
+#ifdef HAVE_XFIXES
   bool setCursor();
+#endif
   rfb::ScreenSet computeScreenLayout();
 };
 
