@@ -38,6 +38,9 @@
 #include <rfb/Timer.h>
 #include <network/TcpSocket.h>
 #include <network/UnixSocket.h>
+#ifdef HAVE_LIBSYSTEMD
+#  include <systemd/sd-daemon.h>
+#endif
 
 #include <signal.h>
 #include <X11/X.h>
@@ -78,7 +81,7 @@ StringParameter interface("interface",
 
 static const char* defaultDesktopName()
 {
-  size_t host_max = sysconf(_SC_HOST_NAME_MAX);
+  long host_max = sysconf(_SC_HOST_NAME_MAX);
   if (host_max < 0)
     return "";
 
@@ -90,7 +93,7 @@ static const char* defaultDesktopName()
   if (pwent == NULL)
     return "";
 
-  size_t len = snprintf(NULL, 0, "%s@%s", pwent->pw_name, hostname.data());
+  int len = snprintf(NULL, 0, "%s@%s", pwent->pw_name, hostname.data());
   if (len < 0)
     return "";
 
@@ -108,11 +111,42 @@ static const char* defaultDesktopName()
 
 static bool caughtSignal = false;
 
-static void CleanupSignalHandler(int sig)
+static void CleanupSignalHandler(int /*sig*/)
 {
   caughtSignal = true;
 }
 
+static bool hasSystemdListeners()
+{
+#ifdef HAVE_LIBSYSTEMD
+  // This also returns true on errors, because we then assume we were
+  // meant to use systemd but failed somewhere
+  return sd_listen_fds(0) != 0;
+#else
+  return false;
+#endif
+}
+
+static int createSystemdListeners(std::list<SocketListener*> *listeners)
+{
+#ifdef HAVE_LIBSYSTEMD
+  int count = sd_listen_fds(0);
+  if (count < 0) {
+    vlog.error("Error getting listening sockets from systemd: %s",
+               strerror(-count));
+    return count;
+  }
+
+  for (int i = 0; i < count; ++i)
+      listeners->push_back(new TcpListener(SD_LISTEN_FDS_START + i));
+
+  return count;
+#else
+  (void)listeners;
+  return 0;
+#endif
+}
+
 
 class FileTcpFilter : public TcpFilter
 {
@@ -252,6 +286,10 @@ int main(int argc, char** argv)
   Configuration::removeParam("SendCutText");
   Configuration::removeParam("MaxCutText");
 
+  // Assume different defaults when socket activated
+  if (hasSystemdListeners())
+    rfbport.setParam(-1);
+
   for (int i = 1; i < argc; i++) {
     if (Configuration::setParam(argv[i]))
       continue;
@@ -275,11 +313,10 @@ int main(int argc, char** argv)
     usage();
   }
 
-  CharArray dpyStr(displayname.getData());
-  if (!(dpy = XOpenDisplay(dpyStr.buf[0] ? dpyStr.buf : 0))) {
+  if (!(dpy = XOpenDisplay(displayname))) {
     // FIXME: Why not vlog.error(...)?
     fprintf(stderr,"%s: unable to open display \"%s\"\r\n",
-            programName, XDisplayName(dpyStr.buf));
+            programName, XDisplayName(displayname));
     exit(1);
   }
 
@@ -301,37 +338,51 @@ int main(int argc, char** argv)
 
     VNCServerST server(desktopName, &desktop);
 
+    if (createSystemdListeners(&listeners) > 0) {
+      // When systemd is in charge of listeners, do not listen to anything else
+      vlog.info("Listening on systemd sockets");
+    }
+
     if (rfbunixpath.getValueStr()[0] != '\0') {
       listeners.push_back(new network::UnixListener(rfbunixpath, rfbunixmode));
       vlog.info("Listening on %s (mode %04o)", (const char*)rfbunixpath, (int)rfbunixmode);
     }
 
     if ((int)rfbport != -1) {
+      std::list<network::SocketListener*> tcp_listeners;
       const char *addr = interface;
+
       if (strcasecmp(addr, "all") == 0)
         addr = 0;
       if (localhostOnly)
-        createLocalTcpListeners(&listeners, (int)rfbport);
+        createLocalTcpListeners(&tcp_listeners, (int)rfbport);
       else
-        createTcpListeners(&listeners, addr, (int)rfbport);
-      vlog.info("Listening for VNC connections on %s interface(s), port %d",
-                localhostOnly ? "local" : (const char*)interface,
-                (int)rfbport);
+        createTcpListeners(&tcp_listeners, addr, (int)rfbport);
+
+      if (!tcp_listeners.empty()) {
+        listeners.splice (listeners.end(), tcp_listeners);
+        vlog.info("Listening for VNC connections on %s interface(s), port %d",
+                  localhostOnly ? "local" : (const char*)interface,
+                  (int)rfbport);
+      }
+
+      FileTcpFilter fileTcpFilter(hostsFile);
+      if (strlen(hostsFile) != 0)
+        for (std::list<SocketListener*>::iterator i = listeners.begin();
+             i != listeners.end();
+             i++)
+          (*i)->setFilter(&fileTcpFilter);
     }
 
-    const char *hostsData = hostsFile.getData();
-    FileTcpFilter fileTcpFilter(hostsData);
-    if (strlen(hostsData) != 0)
-      for (std::list<SocketListener*>::iterator i = listeners.begin();
-           i != listeners.end();
-           i++)
-        (*i)->setFilter(&fileTcpFilter);
-    delete[] hostsData;
+    if (listeners.empty()) {
+      vlog.error("No path or port configured for incoming connections");
+      return -1;
+    }
 
     PollingScheduler sched((int)pollingCycle, (int)maxProcessorUsage);
 
     while (!caughtSignal) {
-      int wait_ms;
+      int wait_ms, nextTimeout;
       struct timeval tv;
       fd_set rfds, wfds;
       std::list<Socket*> sockets;
@@ -366,7 +417,7 @@ int main(int argc, char** argv)
       if (!clients_connected)
         sched.reset();
 
-      wait_ms = 0;
+      wait_ms = -1;
 
       if (sched.isRunning()) {
         wait_ms = sched.millisRemaining();
@@ -375,7 +426,10 @@ int main(int argc, char** argv)
         }
       }
 
-      soonestTimeout(&wait_ms, Timer::checkTimeouts());
+      // Trigger timers and check when the next will expire
+      nextTimeout = Timer::checkTimeouts();
+      if (nextTimeout >= 0 && (wait_ms == -1 || nextTimeout < wait_ms))
+        wait_ms = nextTimeout;
 
       tv.tv_sec = wait_ms / 1000;
       tv.tv_usec = (wait_ms % 1000) * 1000;
@@ -383,7 +437,7 @@ int main(int argc, char** argv)
       // Do the wait...
       sched.sleepStarted();
       int n = select(FD_SETSIZE, &rfds, &wfds, 0,
-                     wait_ms ? &tv : NULL);
+                     wait_ms >= 0 ? &tv : NULL);
       sched.sleepFinished();
 
       if (n < 0) {
