@@ -28,6 +28,7 @@
 #include <rfb/LogWriter.h>
 #include <rfb/Logger_stdio.h>
 #include <rfb/Logger_syslog.h>
+#include <rfb/util.h>
 
 #include "RFBGlue.h"
 
@@ -113,9 +114,7 @@ int vncSetParamSimple(const char *nameAndValue)
 
 char* vncGetParam(const char *name)
 {
-  rfb::VoidParameter *param;
-  char *value;
-  char *ret;
+  VoidParameter *param;
 
   // Hack to avoid exposing password!
   if (strcasecmp(name, "Password") == 0)
@@ -125,15 +124,7 @@ char* vncGetParam(const char *name)
   if (param == NULL)
     return NULL;
 
-  value = param->getValueStr();
-  if (value == NULL)
-    return NULL;
-
-  ret = strdup(value);
-
-  delete [] value;
-
-  return ret;
+  return strdup(param->getValueStr().c_str());
 }
 
 const char* vncGetParamDesc(const char *name)
@@ -234,7 +225,7 @@ int vncIsTCPPortUsed(int port)
 char* vncConvertLF(const char* src, size_t bytes)
 {
   try {
-    return convertLF(src, bytes);
+    return strdup(convertLF(src, bytes).c_str());
   } catch (...) {
     return NULL;
   }
@@ -243,7 +234,7 @@ char* vncConvertLF(const char* src, size_t bytes)
 char* vncLatin1ToUTF8(const char* src, size_t bytes)
 {
   try {
-    return latin1ToUTF8(src, bytes);
+    return strdup(latin1ToUTF8(src, bytes).c_str());
   } catch (...) {
     return NULL;
   }
@@ -252,13 +243,17 @@ char* vncLatin1ToUTF8(const char* src, size_t bytes)
 char* vncUTF8ToLatin1(const char* src, size_t bytes)
 {
   try {
-    return utf8ToLatin1(src, bytes);
+    return strdup(utf8ToLatin1(src, bytes).c_str());
   } catch (...) {
     return NULL;
   }
 }
 
-void vncStrFree(char* str)
+int vncIsValidUTF8(const char* str, size_t bytes)
 {
-  strFree(str);
+  try {
+    return isValidUTF8(str, bytes);
+  } catch (...) {
+    return 0;
+  }
 }
