@@ -1,5 +1,5 @@
 /* Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
- * Copyright 2011-2019 Pierre Ossman for Cendio AB
+ * Copyright 2011-2024 Pierre Ossman for Cendio AB
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,6 @@
 #include <rfb/Configuration.h>
 #include <rfb/Logger_stdio.h>
 #include <rfb/LogWriter.h>
-#include <rfb/util.h>
 #include <rfb/ServerCore.h>
 #include <rdr/HexOutStream.h>
 #include <rfb/LogWriter.h>
@@ -230,6 +229,7 @@ void vncExtensionInit(void)
         }
 
         if (!inetd && rfbport != -1) {
+          std::list<network::SocketListener*> tcp_listeners;
           const char *addr = interface;
           int port = rfbport;
           if (port == 0) port = 5900 + atoi(vncGetDisplay());
@@ -237,22 +237,27 @@ void vncExtensionInit(void)
           if (strcasecmp(addr, "all") == 0)
             addr = 0;
           if (localhostOnly)
-            network::createLocalTcpListeners(&listeners, port);
+            network::createLocalTcpListeners(&tcp_listeners, port);
           else
-            network::createTcpListeners(&listeners, addr, port);
+            network::createTcpListeners(&tcp_listeners, addr, port);
 
-          vlog.info("Listening for VNC connections on %s interface(s), port %d",
-                    localhostOnly ? "local" : (const char*)interface,
-                    port);
+          if (!tcp_listeners.empty()) {
+            listeners.splice (listeners.end(), tcp_listeners);
+            vlog.info("Listening for VNC connections on %s interface(s), port %d",
+                      localhostOnly ? "local" : (const char*)interface,
+                      port);
+          }
         }
 
-        CharArray desktopNameStr(desktopName.getData());
+        if (!inetd && listeners.empty())
+          throw rdr::Exception("No path or port configured for incoming connections");
+
         PixelFormat pf = vncGetPixelFormat(scr);
 
         vncSetGlueContext(scr);
         desktop[scr] = new XserverDesktop(scr,
                                           listeners,
-                                          desktopNameStr.buf,
+                                          desktopName,
                                           pf,
                                           vncGetScreenWidth(),
                                           vncGetScreenHeight(),
@@ -262,7 +267,7 @@ void vncExtensionInit(void)
 
         if (scr == 0 && vncInetdSock != -1 && listeners.empty()) {
           network::Socket* sock = new network::TcpSocket(vncInetdSock);
-          desktop[scr]->addClient(sock, false);
+          desktop[scr]->addClient(sock, false, false);
           vlog.info("added inetd sock");
         }
       }
@@ -338,7 +343,7 @@ void vncSendClipboardData(const char* data)
     desktop[scr]->sendClipboardData(data);
 }
 
-int vncConnectClient(const char *addr)
+int vncConnectClient(const char *addr, int viewOnly)
 {
   if (strlen(addr) == 0) {
     try {
@@ -350,15 +355,16 @@ int vncConnectClient(const char *addr)
     return 0;
   }
 
-  char *host;
+  std::string host;
   int port;
 
   getHostAndPort(addr, &host, &port, 5500);
 
   try {
-    network::Socket* sock = new network::TcpSocket(host, port);
-    delete [] host;
-    desktop[0]->addClient(sock, true);
+    network::Socket* sock = new network::TcpSocket(host.c_str(), port);
+    vlog.info("Reverse connection: %s:%d%s", host.c_str(), port,
+              viewOnly ? " (view only)" : "");
+    desktop[0]->addClient(sock, true, (bool)viewOnly);
   } catch (rdr::Exception& e) {
     vlog.error("Reverse connection: %s",e.str());
     return -1;
@@ -478,6 +484,33 @@ void vncRefreshScreenLayout(int scrIdx)
   }
 }
 
+uint64_t vncGetMsc(int scrIdx)
+{
+  try {
+    return desktop[scrIdx]->getMsc();
+  } catch (rdr::Exception& e) {
+    vncFatalError("vncGetMsc: %s\n", e.str());
+  }
+}
+
+void vncQueueMsc(int scrIdx, uint64_t id, uint64_t msc)
+{
+  try {
+    desktop[scrIdx]->queueMsc(id, msc);
+  } catch (rdr::Exception& e) {
+    vncFatalError("vncQueueMsc: %s\n", e.str());
+  }
+}
+
+void vncAbortMsc(int scrIdx, uint64_t id)
+{
+  try {
+    desktop[scrIdx]->abortMsc(id);
+  } catch (rdr::Exception& e) {
+    vncFatalError("vncAbortMsc: %s\n", e.str());
+  }
+}
+
 int vncOverrideParam(const char *nameAndValue)
 {
   const char* equalSign = strchr(nameAndValue, '=');
