@@ -28,6 +28,9 @@
 #include "vncExtInit.h"
 #include "RFBGlue.h"
 
+/* This is a C header, so safe to include */
+#include <rfb/KeysymStr.h>
+
 #include "inputstr.h"
 #include "inpututils.h"
 #include "mi.h"
@@ -404,7 +407,8 @@ static void vncKeysymKeyboardEvent(KeySym keysym, int down)
 		 * This can happen quite often as we ignore some
 		 * key presses.
 		 */
-		LOG_DEBUG("Unexpected release of keysym 0x%x", keysym);
+		LOG_DEBUG("Unexpected release of keysym XK_%s (0x%04x)",
+                          KeySymName(keysym), keysym);
 		return;
 	}
 
@@ -466,12 +470,13 @@ static void vncKeysymKeyboardEvent(KeySym keysym, int down)
 	if (keycode == 0) {
 		keycode = vncAddKeysym(keysym, state);
 		if (keycode == 0) {
-			LOG_ERROR("Failure adding new keysym 0x%x", keysym);
+			LOG_ERROR("Failure adding new keysym XK_%s (0x%x)",
+                                  KeySymName(keysym), keysym);
 			return;
 		}
 
-		LOG_INFO("Added unknown keysym 0x%x to keycode %d",
-		         keysym, keycode);
+		LOG_INFO("Added keysym XK_%s (0x%04x) to keycode %d",
+		         KeySymName(keysym), keysym, keycode);
 
 		/*
 		 * The state given to addKeysym() is just a hint and
@@ -480,7 +485,8 @@ static void vncKeysymKeyboardEvent(KeySym keysym, int down)
 		 */
 		keycode = vncKeysymToKeycode(keysym, state, &new_state);
 		if (keycode == 0) {
-			LOG_ERROR("Newly added keysym 0x%x cannot be generated", keysym);
+			LOG_ERROR("Newly added keysym XK_%s (0x%x) cannot be generated",
+                                  KeySymName(keysym), keysym);
 			return;
 		}
 	}
@@ -501,7 +507,8 @@ static void vncKeysymKeyboardEvent(KeySym keysym, int down)
 	    	KeyCode keycode2;
 	    	unsigned new_state2;
 
-		LOG_DEBUG("Finding alternative to keysym 0x%x to avoid fake shift for numpad", keysym);
+		LOG_DEBUG("Finding alternative to keysym XK_%s (0x%x) to avoid fake shift for numpad",
+                          KeySymName(keysym), keysym);
 
 		for (i = 0;i < sizeof(altKeysym)/sizeof(altKeysym[0]);i++) {
 			KeySym altsym;
@@ -605,12 +612,16 @@ static void vncKeysymKeyboardEvent(KeySym keysym, int down)
 	/* Now press the actual key */
 	pressKey(vncKeyboardDev, keycode, TRUE, "keycode");
 
+	if(down)
+		vncOnKeyUsed(keycode);
+
 	/* And store the mapping so that we can do a proper release later */
 	for (i = 0;i < 256;i++) {
 		if (i == keycode)
 			continue;
 		if (pressedKeys[i] == keysym) {
-			LOG_ERROR("Keysym 0x%x generated by both keys %d and %d", keysym, i, keycode);
+			LOG_ERROR("Keysym XK_%s (0x%04x) generated by both keys %d and %d",
+                                  KeySymName(keysym), keysym, i, keycode);
 			pressedKeys[i] = NoSymbol;
 		}
 	}
