@@ -32,7 +32,6 @@
 #include <rfb/Hostname.h>
 #include <rfb/LogWriter.h>
 #include <rfb/Security.h>
-#include <rfb/util.h>
 #include <rfb/screenTypes.h>
 #include <rfb/fenceTypes.h>
 #include <rfb/Timer.h>
@@ -56,9 +55,7 @@
 #include "win32.h"
 #endif
 
-using namespace rdr;
 using namespace rfb;
-using namespace std;
 
 static rfb::LogWriter vlog("CConn");
 
@@ -76,8 +73,7 @@ static const PixelFormat mediumColourPF(8, 8, false, true,
 static const unsigned bpsEstimateWindow = 1000;
 
 CConn::CConn(const char* vncServerName, network::Socket* socket=NULL)
-  : serverHost(0), serverPort(0), desktop(NULL),
-    updateCount(0), pixelCount(0),
+  : serverPort(0), desktop(NULL), updateCount(0), pixelCount(0),
     lastServerEncoding((unsigned int)-1), bpsEstimate(20000000)
 {
   setShared(::shared);
@@ -100,14 +96,15 @@ CConn::CConn(const char* vncServerName, network::Socket* socket=NULL)
       if (strchr(vncServerName, '/') != NULL) {
         sock = new network::UnixSocket(vncServerName);
         serverHost = sock->getPeerAddress();
-        vlog.info(_("Connected to socket %s"), serverHost);
+        vlog.info(_("Connected to socket %s"), serverHost.c_str());
       } else
 #endif
       {
         getHostAndPort(vncServerName, &serverHost, &serverPort);
 
-        sock = new network::TcpSocket(serverHost, serverPort);
-        vlog.info(_("Connected to host %s port %d"), serverHost, serverPort);
+        sock = new network::TcpSocket(serverHost.c_str(), serverPort);
+        vlog.info(_("Connected to host %s port %d"),
+                  serverHost.c_str(), serverPort);
       }
     } catch (rdr::Exception& e) {
       vlog.error("%s", e.str());
@@ -119,7 +116,7 @@ CConn::CConn(const char* vncServerName, network::Socket* socket=NULL)
 
   Fl::add_fd(sock->getFd(), FL_READ | FL_EXCEPT, socketEvent, this);
 
-  setServerName(serverHost);
+  setServerName(serverHost.c_str());
   setStreams(&sock->inStream(), &sock->outStream());
 
   initialiseProtocol();
@@ -137,7 +134,6 @@ CConn::~CConn()
   if (desktop)
     delete desktop;
 
-  delete [] serverHost;
   if (sock)
     Fl::remove_fd(sock->getFd());
   delete sock;
@@ -161,7 +157,7 @@ const char *CConn::connectionInfo()
   strcat(infoText, "\n");
 
   snprintf(scratch, sizeof(scratch),
-           _("Host: %.80s port: %d"), serverHost, serverPort);
+           _("Host: %.80s port: %d"), serverHost.c_str(), serverPort);
   strcat(infoText, scratch);
   strcat(infoText, "\n");
 
@@ -284,9 +280,7 @@ void CConn::socketEvent(FL_SOCKET fd, void *data)
     when |= FL_WRITE;
 
   Fl::add_fd(fd, when, socketEvent, data);
-
   recursing = false;
-  Fl::add_fd(fd, FL_READ | FL_EXCEPT, socketEvent, data);
 }
 
 ////////////////////// CConnection callback methods //////////////////////
@@ -399,7 +393,8 @@ void CConn::framebufferUpdateEnd()
 
 // The rest of the callbacks are fairly self-explanatory...
 
-void CConn::setColourMapEntries(int firstColour, int nColours, rdr::U16* rgbs)
+void CConn::setColourMapEntries(int /*firstColour*/, int /*nColours*/,
+                                uint16_t* /*rgbs*/)
 {
   vlog.error(_("Invalid SetColourMapEntries from server!"));
 }
@@ -425,7 +420,7 @@ bool CConn::dataRect(const Rect& r, int encoding)
 }
 
 void CConn::setCursor(int width, int height, const Point& hotspot,
-                      const rdr::U8* data)
+                      const uint8_t* data)
 {
   desktop->setCursor(width, height, hotspot, data);
 }
@@ -435,7 +430,7 @@ void CConn::setCursorPos(const Point& pos)
   desktop->setCursorPos(pos);
 }
 
-void CConn::fence(rdr::U32 flags, unsigned len, const char data[])
+void CConn::fence(uint32_t flags, unsigned len, const uint8_t data[])
 {
   CMsgHandler::fence(flags, len, data);
 
