@@ -21,6 +21,8 @@
 #include <config.h>
 #endif
 
+#include <algorithm>
+
 #include <assert.h>
 #include <stdio.h>
 #include <string.h>
@@ -28,6 +30,7 @@
 
 #include <rfb/LogWriter.h>
 #include <rfb/CMsgWriter.h>
+#include <rfb/util.h>
 
 #include "DesktopWindow.h"
 #include "OptionsDialog.h"
@@ -145,6 +148,29 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
     }
   }
 
+  // Many window managers don't properly resize overly large windows,
+  // so we'll have to do some sanity checks ourselves here
+  int sx, sy, sw, sh;
+  if (force_position()) {
+    Fl::screen_work_area(sx, sy, sw, sh, geom_x, geom_y);
+  } else {
+    int mx, my;
+
+    // If we don't explicitly request a position then we don't know which
+    // monitor the window manager might place us on. Assume the popular
+    // behaviour of following the cursor.
+
+    Fl::get_mouse(mx, my);
+    Fl::screen_work_area(sx, sy, sw, sh, mx, my);
+  }
+  if ((w > sw) || (h > sh)) {
+    vlog.info(_("Reducing window size to fit on current monitor"));
+    if (w > sw)
+      w = sw;
+    if (h > sh)
+      h = sh;
+  }
+
 #ifdef __APPLE__
   // On OS X we can do the maximize thing properly before the
   // window is showned. Other platforms handled further down...
@@ -219,6 +245,11 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
 
 DesktopWindow::~DesktopWindow()
 {
+  // Don't leave any dangling grabs as they are not automatically
+  // cleaned up on all platforms
+  ungrabPointer();
+  ungrabKeyboard();
+
   // Unregister all timeouts in case they get a change tro trigger
   // again later when this object is already gone.
   Fl::remove_timeout(handleGrab, this);
@@ -256,12 +287,11 @@ const rfb::PixelFormat &DesktopWindow::getPreferredPF()
 
 void DesktopWindow::setName(const char *name)
 {
-  CharArray windowNameStr;
-  windowNameStr.replaceBuf(new char[256]);
+  char windowNameStr[256];
 
-  snprintf(windowNameStr.buf, 256, "%.240s - TigerVNC", name);
+  snprintf(windowNameStr, 256, "%.240s - TigerVNC", name);
 
-  copy_label(windowNameStr.buf);
+  copy_label(windowNameStr);
 }
 
 
@@ -351,7 +381,7 @@ void DesktopWindow::resizeFramebuffer(int new_w, int new_h)
 
 void DesktopWindow::setCursor(int width, int height,
                               const rfb::Point& hotspot,
-                              const rdr::U8* data)
+                              const uint8_t* data)
 {
   viewport->setCursor(width, height, hotspot, data);
 }
@@ -612,12 +642,17 @@ void DesktopWindow::resize(int x, int y, int w, int h)
 
         Fl::screen_xywh(sx, sy, sw, sh, i);
 
-        if ((sx == x) && (sy == y) && (sw == w) && (sh == h)) {
-          vlog.info(_("Adjusting window size to avoid accidental full screen request"));
-          // Assume a panel of some form and adjust the height
-          y += 20;
-          h -= 40;
-        }
+        // We can't trust x and y if the window isn't mapped as the
+        // window manager might adjust those numbers
+        if (shown() && ((sx != x) || (sy != y)))
+            continue;
+
+        if ((sw != w) || (sh != h))
+            continue;
+
+        vlog.info(_("Adjusting window size to avoid accidental full-screen request"));
+        // Assume a panel of some form and adjust the height
+        h -= 40;
       }
     }
   }
@@ -759,6 +794,7 @@ void DesktopWindow::setOverlay(const char* text, ...)
   gettimeofday(&overlayStart, NULL);
 
   delete image;
+  delete [] buffer;
 
   Fl::add_timeout(1.0/60, updateOverlay, this);
 }
@@ -809,15 +845,28 @@ int DesktopWindow::handle(int event)
   case FL_ENTER:
       if (keyboardGrabbed)
           grabPointer();
+      /* fall through */
   case FL_LEAVE:
   case FL_DRAG:
   case FL_MOVE:
-    // We don't get FL_LEAVE with a grabbed pointer, so check manually
     if (mouseGrabbed) {
+      // We don't get FL_LEAVE with a grabbed pointer, so check manually
       if ((Fl::event_x() < 0) || (Fl::event_x() >= w()) ||
           (Fl::event_y() < 0) || (Fl::event_y() >= h())) {
         ungrabPointer();
       }
+#if !defined(WIN32) && !defined(__APPLE__)
+      Window root, child;
+      int x, y, wx, wy;
+      unsigned int mask;
+
+      // We also don't get sensible coordinates on zaphod setups
+      if (XQueryPointer(fl_display, fl_xid(this), &root, &child,
+                        &x, &y, &wx, &wy, &mask) &&
+          (root != XRootWindow(fl_display, fl_screen))) {
+        ungrabPointer();
+      }
+#endif
     }
     if (fullscreen_active()) {
       // calculate width of "edge" regions
@@ -991,7 +1040,8 @@ void DesktopWindow::fullscreen_on()
   fullscreen_screens(top, bottom, left, right);
 #ifdef __APPLE__
   // This is a workaround for a bug in FLTK, see: https://github.com/fltk/fltk/pull/277
-  cocoa_set_level(this, savedLevel);
+  if (cocoa_get_level(this) != savedLevel)
+    cocoa_set_level(this, savedLevel);
 #endif
 
   if (!fullscreen_active())
@@ -999,7 +1049,8 @@ void DesktopWindow::fullscreen_on()
 }
 
 #if !defined(WIN32) && !defined(__APPLE__)
-Bool eventIsFocusWithSerial(Display *display, XEvent *event, XPointer arg)
+Bool eventIsFocusWithSerial(Display* /*display*/, XEvent *event,
+                            XPointer arg)
 {
   unsigned long serial;
 
@@ -1238,7 +1289,7 @@ void DesktopWindow::handleResizeTimeout(void *data)
 }
 
 
-void DesktopWindow::reconfigureFullscreen(void *data)
+void DesktopWindow::reconfigureFullscreen(void* /*data*/)
 {
   std::set<DesktopWindow *>::iterator iter;
 
@@ -1287,7 +1338,7 @@ void DesktopWindow::remoteResize(int width, int height)
     layout.begin()->dimensions.br.y = height;
   } else {
     int i;
-    rdr::U32 id;
+    uint32_t id;
     int sx, sy, sw, sh;
     rfb::Rect viewport_rect, screen_rect;
 
@@ -1314,13 +1365,15 @@ void DesktopWindow::remoteResize(int width, int height)
       sx -= viewport_rect.tl.x;
       sy -= viewport_rect.tl.y;
 
-      // Look for perfectly matching existing screen...
+      // Look for perfectly matching existing screen that is not yet present in
+      // in the screen layout...
       for (iter = cc->server.screenLayout().begin();
            iter != cc->server.screenLayout().end(); ++iter) {
         if ((iter->dimensions.tl.x == sx) &&
             (iter->dimensions.tl.y == sy) &&
             (iter->dimensions.width() == sw) &&
-            (iter->dimensions.height() == sh))
+            (iter->dimensions.height() == sh) &&
+            (std::find(layout.begin(), layout.end(), *iter) == layout.end()))
           break;
       }
 
@@ -1358,15 +1411,17 @@ void DesktopWindow::remoteResize(int width, int height)
       (layout == cc->server.screenLayout()))
     return;
 
-  char buffer[2048];
   vlog.debug("Requesting framebuffer resize from %dx%d to %dx%d",
              cc->server.width(), cc->server.height(), width, height);
-  layout.print(buffer, sizeof(buffer));
-  vlog.debug("%s", buffer);
 
+  char buffer[2048];
+  layout.print(buffer, sizeof(buffer));
   if (!layout.validate(width, height)) {
     vlog.error(_("Invalid screen layout computed for resize request!"));
+    vlog.error("%s", buffer);
     return;
+  } else {
+    vlog.debug("%s", buffer);
   }
 
   cc->writer()->writeSetDesktopSize(width, height, layout);
@@ -1459,7 +1514,7 @@ void DesktopWindow::repositionWidgets()
   vscroll->value(vscroll->clamp(vscroll->value()));
 }
 
-void DesktopWindow::handleClose(Fl_Widget *wnd, void *data)
+void DesktopWindow::handleClose(Fl_Widget* /*wnd*/, void* /*data*/)
 {
   disconnect();
 }
@@ -1516,7 +1571,7 @@ void DesktopWindow::scrollTo(int x, int y)
   damage(FL_DAMAGE_SCROLL);
 }
 
-void DesktopWindow::handleScroll(Fl_Widget *widget, void *data)
+void DesktopWindow::handleScroll(Fl_Widget* /*widget*/, void *data)
 {
   DesktopWindow *self = (DesktopWindow *)data;
 
@@ -1673,14 +1728,12 @@ void DesktopWindow::handleStatsTimeout(void *data)
   fl_draw(buffer, 5, statsHeight - 5);
 
   fl_color(FL_YELLOW);
-  siPrefix(self->stats[statsCount-1].pps, "pix/s",
-           buffer, sizeof(buffer), 3);
-  fl_draw(buffer, 5 + (statsWidth-10)/3, statsHeight - 5);
+  fl_draw(siPrefix(self->stats[statsCount-1].pps, "pix/s").c_str(),
+          5 + (statsWidth-10)/3, statsHeight - 5);
 
   fl_color(FL_RED);
-  siPrefix(self->stats[statsCount-1].bps * 8, "bps",
-           buffer, sizeof(buffer), 3);
-  fl_draw(buffer, 5 + (statsWidth-10)*2/3, statsHeight - 5);
+  fl_draw(siPrefix(self->stats[statsCount-1].bps * 8, "bps").c_str(),
+          5 + (statsWidth-10)*2/3, statsHeight - 5);
 
   image = surface->image();
   delete surface;
