@@ -24,7 +24,6 @@
 #include <stdlib.h>
 #include <list>
 
-#include <rdr/types.h>
 #include <rfb/encodings.h>
 
 #if defined(HAVE_GNUTLS) || defined(HAVE_NETTLE)
@@ -36,11 +35,14 @@
 #endif
 
 #include "OptionsDialog.h"
-#include "fltk_layout.h"
 #include "i18n.h"
 #include "menukey.h"
 #include "parameters.h"
-#include "MonitorArrangement.h"
+
+#include "fltk/layout.h"
+#include "fltk/util.h"
+#include "fltk/Fl_Monitor_Arrangement.h"
+#include "fltk/Fl_Navigation.h"
 
 #include <FL/Fl.H>
 #include <FL/Fl_Tabs.H>
@@ -52,7 +54,6 @@
 #include <FL/Fl_Choice.H>
 
 using namespace std;
-using namespace rdr;
 using namespace rfb;
 
 std::map<OptionsCallback*, void*> OptionsDialog::callbacks;
@@ -60,19 +61,26 @@ std::map<OptionsCallback*, void*> OptionsDialog::callbacks;
 static std::set<OptionsDialog *> instances;
 
 OptionsDialog::OptionsDialog()
-  : Fl_Window(450, 460, _("VNC Viewer: Connection Options"))
+  : Fl_Window(580, 420, _("TigerVNC Options"))
 {
   int x, y;
+  Fl_Navigation *navigation;
   Fl_Button *button;
 
-  Fl_Tabs *tabs = new Fl_Tabs(OUTER_MARGIN, OUTER_MARGIN,
-                             w() - OUTER_MARGIN*2,
-                             h() - OUTER_MARGIN*2 - INNER_MARGIN - BUTTON_HEIGHT);
-
+  // Odd dimensions to get rid of extra borders
+  // FIXME: We need to retain the top border on Windows as they don't
+  //        have any separator for the caption, which looks odd
+#ifdef WIN32
+  navigation = new Fl_Navigation(-1, 0, w()+2,
+                                 h() - OUTER_MARGIN - BUTTON_HEIGHT - OUTER_MARGIN);
+#else
+  navigation = new Fl_Navigation(-1, -1, w()+2,
+                                 h()+1 - OUTER_MARGIN - BUTTON_HEIGHT - OUTER_MARGIN);
+#endif
   {
     int tx, ty, tw, th;
 
-    tabs->client_area(tx, ty, tw, th, TABS_HEIGHT);
+    navigation->client_area(tx, ty, tw, th, 150);
 
     createCompressionPage(tx, ty, tw, th);
     createSecurityPage(tx, ty, tw, th);
@@ -81,7 +89,7 @@ OptionsDialog::OptionsDialog()
     createMiscPage(tx, ty, tw, th);
   }
 
-  tabs->end();
+  navigation->end();
 
   x = w() - BUTTON_WIDTH * 2 - INNER_MARGIN - OUTER_MARGIN;
   y = h() - BUTTON_HEIGHT - OUTER_MARGIN;
@@ -209,11 +217,11 @@ void OptionsDialog::loadOptions(void)
   /* Security */
   Security security(SecurityClient::secTypes);
 
-  list<U8> secTypes;
-  list<U8>::iterator iter;
+  list<uint8_t> secTypes;
+  list<uint8_t>::iterator iter;
 
-   list<U32> secTypesExt;
-   list<U32>::iterator iterExt;
+   list<uint32_t> secTypesExt;
+   list<uint32_t>::iterator iterExt;
 
   encNoneCheckbox->value(false);
 #ifdef HAVE_GNUTLS
@@ -279,9 +287,16 @@ void OptionsDialog::loadOptions(void)
     case secTypeRA2:
     case secTypeRA256:
       encRSAAESCheckbox->value(true);
+      authVncCheckbox->value(true);
+      authPlainCheckbox->value(true);
+      break;
     case secTypeRA2ne:
     case secTypeRAne256:
       authVncCheckbox->value(true);
+      /* fall through */
+    case secTypeDH:
+    case secTypeMSLogonII:
+      encNoneCheckbox->value(true);
       authPlainCheckbox->value(true);
       break;
 #endif
@@ -332,7 +347,7 @@ void OptionsDialog::loadOptions(void)
     }
   }
 
-  monitorArrangement->set(fullScreenSelectedMonitors.getParam());
+  monitorArrangement->value(fullScreenSelectedMonitors.getParam());
 
   handleFullScreenMode(selectedMonitorsButton, this);
 
@@ -385,6 +400,7 @@ void OptionsDialog::storeOptions(void)
     if (authVncCheckbox->value()) {
       security.EnableSecType(secTypeVncAuth);
 #ifdef HAVE_NETTLE
+      security.EnableSecType(secTypeRA2ne);
       security.EnableSecType(secTypeRAne256);
 #endif
     }
@@ -393,6 +409,8 @@ void OptionsDialog::storeOptions(void)
 #ifdef HAVE_NETTLE
       security.EnableSecType(secTypeRA2ne);
       security.EnableSecType(secTypeRAne256);
+      security.EnableSecType(secTypeDH);
+      security.EnableSecType(secTypeMSLogonII);
 #endif
     }
   }
@@ -465,7 +483,7 @@ void OptionsDialog::storeOptions(void)
     }
   }
 
-  fullScreenSelectedMonitors.setParam(monitorArrangement->get());
+  fullScreenSelectedMonitors.setParam(monitorArrangement->value());
 
   /* Misc. */
   shared.setParam(sharedCheckbox->value());
@@ -486,7 +504,6 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
   int orig_tx, orig_ty;
   int col1_ty, col2_ty;
   int half_width, full_width;
-  int height;
 
   tx += OUTER_MARGIN;
   ty += OUTER_MARGIN;
@@ -508,18 +525,15 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
 
   /* VNC encoding box */
   ty += GROUP_LABEL_OFFSET;
-  height = GROUP_MARGIN * 2 + TIGHT_MARGIN * 3 + RADIO_HEIGHT * 4;
-#ifdef HAVE_H264
-  height += TIGHT_MARGIN + RADIO_HEIGHT;
-#endif
-  encodingGroup = new Fl_Group(tx, ty, half_width, height,
+  encodingGroup = new Fl_Group(tx, ty, half_width, 0,
                                 _("Preferred encoding"));
-  encodingGroup->box(FL_ENGRAVED_BOX);
+  encodingGroup->box(FL_FLAT_BOX);
+  encodingGroup->labelfont(FL_BOLD);
   encodingGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
 
   {
-    tx += GROUP_MARGIN;
-    ty += GROUP_MARGIN;
+    tx += INDENT;
+    ty += TIGHT_MARGIN;
 
     tightButton = new Fl_Round_Button(LBLRIGHT(tx, ty,
                                                RADIO_MIN_WIDTH,
@@ -559,9 +573,12 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
     ty += RADIO_HEIGHT + TIGHT_MARGIN;
   }
 
-  ty += GROUP_MARGIN - TIGHT_MARGIN;
+  ty -= TIGHT_MARGIN;
 
   encodingGroup->end();
+  /* Needed for resize to work sanely */
+  encodingGroup->resizable(NULL);
+  encodingGroup->size(encodingGroup->w(), ty - encodingGroup->y());
   col1_ty = ty;
 
   /* Second column */
@@ -570,14 +587,14 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
 
   /* Color box */
   ty += GROUP_LABEL_OFFSET;
-  height = GROUP_MARGIN * 2 + TIGHT_MARGIN * 3 + RADIO_HEIGHT * 4;
-  colorlevelGroup = new Fl_Group(tx, ty, half_width, height, _("Color level"));
-  colorlevelGroup->box(FL_ENGRAVED_BOX);
+  colorlevelGroup = new Fl_Group(tx, ty, half_width, 0, _("Color level"));
+  colorlevelGroup->labelfont(FL_BOLD);
+  colorlevelGroup->box(FL_FLAT_BOX);
   colorlevelGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
 
   {
-    tx += GROUP_MARGIN;
-    ty += GROUP_MARGIN;
+    tx += INDENT;
+    ty += TIGHT_MARGIN;
 
     fullcolorCheckbox = new Fl_Round_Button(LBLRIGHT(tx, ty,
                                                      RADIO_MIN_WIDTH,
@@ -608,9 +625,13 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
     ty += RADIO_HEIGHT + TIGHT_MARGIN;
   }
 
-  ty += GROUP_MARGIN - TIGHT_MARGIN;
+  ty -= TIGHT_MARGIN;
 
   colorlevelGroup->end();
+  /* Needed for resize to work sanely */
+  colorlevelGroup->resizable(NULL);
+  colorlevelGroup->size(colorlevelGroup->w(),
+                        ty - colorlevelGroup->y());
   col2_ty = ty;
 
   /* Back to normal */
@@ -622,6 +643,7 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
                                                      CHECK_MIN_WIDTH,
                                                      CHECK_HEIGHT,
                                                      _("Custom compression level:")));
+  compressionCheckbox->labelfont(FL_BOLD);
   compressionCheckbox->callback(handleCompression, this);
   ty += CHECK_HEIGHT + TIGHT_MARGIN;
 
@@ -635,6 +657,7 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
                                               CHECK_MIN_WIDTH,
                                               CHECK_HEIGHT,
                                               _("Allow JPEG compression:")));
+  jpegCheckbox->labelfont(FL_BOLD);
   jpegCheckbox->callback(handleJpeg, this);
   ty += CHECK_HEIGHT + TIGHT_MARGIN;
 
@@ -654,7 +677,7 @@ void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
   Fl_Group *group = new Fl_Group(tx, ty, tw, th, _("Security"));
 
   int orig_tx;
-  int width, height;
+  int width;
 
   tx += OUTER_MARGIN;
   ty += OUTER_MARGIN;
@@ -665,21 +688,14 @@ void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
 
   /* Encryption */
   ty += GROUP_LABEL_OFFSET;
-
-#if defined(HAVE_GNUTLS) && defined(HAVE_NETTLE)
-  height = GROUP_MARGIN * 2 + TIGHT_MARGIN * 5 + CHECK_HEIGHT * 4 + (INPUT_LABEL_OFFSET + INPUT_HEIGHT) * 2;
-#elif defined(HAVE_GNUTLS)
-  height = GROUP_MARGIN * 2 + TIGHT_MARGIN * 4 + CHECK_HEIGHT * 3 + (INPUT_LABEL_OFFSET + INPUT_HEIGHT) * 2;
-#elif defined(HAVE_NETTLE)
-  height = GROUP_MARGIN * 2 + TIGHT_MARGIN * 1 + CHECK_HEIGHT * 2;
-#endif
-  encryptionGroup = new Fl_Group(tx, ty, width, height, _("Encryption"));
-  encryptionGroup->box(FL_ENGRAVED_BOX);
+  encryptionGroup = new Fl_Group(tx, ty, width, 0, _("Encryption"));
+  encryptionGroup->labelfont(FL_BOLD);
+  encryptionGroup->box(FL_FLAT_BOX);
   encryptionGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
 
   {
-    tx += GROUP_MARGIN;
-    ty += GROUP_MARGIN;
+    tx += INDENT;
+    ty += TIGHT_MARGIN;
 
     encNoneCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
                                                    CHECK_MIN_WIDTH,
@@ -703,14 +719,14 @@ void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
 
     ty += INPUT_LABEL_OFFSET;
     caInput = new Fl_Input(tx + INDENT, ty, 
-                           width - GROUP_MARGIN*2 - INDENT, INPUT_HEIGHT,
+                           width - INDENT * 2, INPUT_HEIGHT,
                            _("Path to X509 CA certificate"));
     caInput->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
     ty += INPUT_HEIGHT + TIGHT_MARGIN;
 
     ty += INPUT_LABEL_OFFSET;
     crlInput = new Fl_Input(tx + INDENT, ty,
-                            width - GROUP_MARGIN*2 - INDENT, INPUT_HEIGHT,
+                            width - INDENT * 2, INPUT_HEIGHT,
                             _("Path to X509 CRL file"));
     crlInput->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
     ty += INPUT_HEIGHT + TIGHT_MARGIN;
@@ -725,9 +741,13 @@ void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
 #endif
   }
 
-  ty += GROUP_MARGIN - TIGHT_MARGIN;
+  ty -= TIGHT_MARGIN;
 
   encryptionGroup->end();
+  /* Needed for resize to work sanely */
+  encryptionGroup->resizable(NULL);
+  encryptionGroup->size(encryptionGroup->w(),
+                        ty - encryptionGroup->y());
 
   /* Back to normal */
   tx = orig_tx;
@@ -735,14 +755,14 @@ void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
 
   /* Authentication */
   ty += GROUP_LABEL_OFFSET;
-  height = GROUP_MARGIN * 2 + TIGHT_MARGIN * 2 + CHECK_HEIGHT * 3;
-  authenticationGroup = new Fl_Group(tx, ty, width, height, _("Authentication"));
-  authenticationGroup->box(FL_ENGRAVED_BOX);
+  authenticationGroup = new Fl_Group(tx, ty, width, 0, _("Authentication"));
+  authenticationGroup->labelfont(FL_BOLD);
+  authenticationGroup->box(FL_FLAT_BOX);
   authenticationGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
 
   {
-    tx += GROUP_MARGIN;
-    ty += GROUP_MARGIN;
+    tx += INDENT;
+    ty += TIGHT_MARGIN;
 
     authNoneCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
                                                     CHECK_MIN_WIDTH,
@@ -763,9 +783,13 @@ void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
     ty += CHECK_HEIGHT + TIGHT_MARGIN;
   }
 
-  ty += GROUP_MARGIN - TIGHT_MARGIN;
+  ty -= TIGHT_MARGIN;
 
   authenticationGroup->end();
+  /* Needed for resize to work sanely */
+  authenticationGroup->resizable(NULL);
+  authenticationGroup->size(authenticationGroup->w(),
+                            ty - authenticationGroup->y());
 
   /* Back to normal */
   tx = orig_tx;
@@ -792,22 +816,20 @@ void OptionsDialog::createInputPage(int tx, int ty, int tw, int th)
                                                   CHECK_MIN_WIDTH,
                                                   CHECK_HEIGHT,
                                                   _("View only (ignore mouse and keyboard)")));
-  ty += CHECK_HEIGHT + TIGHT_MARGIN;
+  ty += CHECK_HEIGHT + INNER_MARGIN;
 
   orig_tx = tx;
 
   /* Mouse */
   ty += GROUP_LABEL_OFFSET;
   mouseGroup = new Fl_Group(tx, ty, width, 0, _("Mouse"));
-  mouseGroup->box(FL_ENGRAVED_BOX);
+  mouseGroup->labelfont(FL_BOLD);
+  mouseGroup->box(FL_FLAT_BOX);
   mouseGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
 
-  /* Needed for final resize to work sanely */
-  mouseGroup->resizable(NULL);
-
   {
-    tx += GROUP_MARGIN;
-    ty += GROUP_MARGIN;
+    tx += INDENT;
+    ty += TIGHT_MARGIN;
 
     emulateMBCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
                                                      CHECK_MIN_WIDTH,
@@ -821,9 +843,11 @@ void OptionsDialog::createInputPage(int tx, int ty, int tw, int th)
                                                     _("Show dot when no cursor")));
     ty += CHECK_HEIGHT + TIGHT_MARGIN;
   }
-  ty += GROUP_MARGIN - TIGHT_MARGIN;
+  ty -= TIGHT_MARGIN;
 
   mouseGroup->end();
+  /* Needed for resize to work sanely */
+  mouseGroup->resizable(NULL);
   mouseGroup->size(mouseGroup->w(), ty - mouseGroup->y());
 
   /* Back to normal */
@@ -833,15 +857,13 @@ void OptionsDialog::createInputPage(int tx, int ty, int tw, int th)
   /* Keyboard */
   ty += GROUP_LABEL_OFFSET;
   keyboardGroup = new Fl_Group(tx, ty, width, 0, _("Keyboard"));
-  keyboardGroup->box(FL_ENGRAVED_BOX);
+  keyboardGroup->labelfont(FL_BOLD);
+  keyboardGroup->box(FL_FLAT_BOX);
   keyboardGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
 
-  /* Needed for final resize to work sanely */
-  keyboardGroup->resizable(NULL);
-
   {
-    tx += GROUP_MARGIN;
-    ty += GROUP_MARGIN;
+    tx += INDENT;
+    ty += TIGHT_MARGIN;
 
     systemKeysCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
                                                       CHECK_MIN_WIDTH,
@@ -857,9 +879,11 @@ void OptionsDialog::createInputPage(int tx, int ty, int tw, int th)
 
     ty += CHOICE_HEIGHT + TIGHT_MARGIN;
   }
-  ty += GROUP_MARGIN - TIGHT_MARGIN;
+  ty -= TIGHT_MARGIN;
 
   keyboardGroup->end();
+  /* Needed for resize to work sanely */
+  keyboardGroup->resizable(NULL);
   keyboardGroup->size(keyboardGroup->w(), ty - keyboardGroup->y());
 
   /* Back to normal */
@@ -869,15 +893,13 @@ void OptionsDialog::createInputPage(int tx, int ty, int tw, int th)
   /* Clipboard */
   ty += GROUP_LABEL_OFFSET;
   clipboardGroup = new Fl_Group(tx, ty, width, 0, _("Clipboard"));
-  clipboardGroup->box(FL_ENGRAVED_BOX);
+  clipboardGroup->labelfont(FL_BOLD);
+  clipboardGroup->box(FL_FLAT_BOX);
   clipboardGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
 
-  /* Needed for final resize to work sanely */
-  clipboardGroup->resizable(NULL);
-
   {
-    tx += GROUP_MARGIN;
-    ty += GROUP_MARGIN;
+    tx += INDENT;
+    ty += TIGHT_MARGIN;
 
     acceptClipboardCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
                                                            CHECK_MIN_WIDTH,
@@ -909,9 +931,11 @@ void OptionsDialog::createInputPage(int tx, int ty, int tw, int th)
     ty += CHECK_HEIGHT + TIGHT_MARGIN;
 #endif
   }
-  ty += GROUP_MARGIN - TIGHT_MARGIN;
+  ty -= TIGHT_MARGIN;
 
   clipboardGroup->end();
+  /* Needed for resize to work sanely */
+  clipboardGroup->resizable(NULL);
   clipboardGroup->size(clipboardGroup->w(), ty - clipboardGroup->y());
 
   /* Back to normal */
@@ -939,16 +963,14 @@ void OptionsDialog::createDisplayPage(int tx, int ty, int tw, int th)
   /* Display mode */
   ty += GROUP_LABEL_OFFSET;
   displayModeGroup = new Fl_Group(tx, ty, width, 0, _("Display mode"));
-  displayModeGroup->box(FL_ENGRAVED_BOX);
+  displayModeGroup->labelfont(FL_BOLD);
+  displayModeGroup->box(FL_FLAT_BOX);
   displayModeGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
 
-  /* Needed for final resize to work sanely */
-  displayModeGroup->resizable(NULL);
-
   {
-    tx += GROUP_MARGIN;
-    ty += GROUP_MARGIN;
-    width -= GROUP_MARGIN * 2;
+    tx += INDENT;
+    ty += TIGHT_MARGIN;
+    width -= INDENT;
 
     windowedButton = new Fl_Round_Button(LBLRIGHT(tx, ty,
                                                   RADIO_MIN_WIDTH,
@@ -982,14 +1004,16 @@ void OptionsDialog::createDisplayPage(int tx, int ty, int tw, int th)
     selectedMonitorsButton->callback(handleFullScreenMode, this);
     ty += RADIO_HEIGHT + TIGHT_MARGIN;
 
-    monitorArrangement = new MonitorArrangement(
+    monitorArrangement = new Fl_Monitor_Arrangement(
                               tx + INDENT, ty,
                               width - INDENT, 150);
     ty += 150 + TIGHT_MARGIN;
   }
-  ty += GROUP_MARGIN - TIGHT_MARGIN;
+  ty -= TIGHT_MARGIN;
 
   displayModeGroup->end();
+  /* Needed for resize to work sanely */
+  displayModeGroup->resizable(NULL);
   displayModeGroup->size(displayModeGroup->w(),
                          ty - displayModeGroup->y());
 
@@ -1004,7 +1028,7 @@ void OptionsDialog::createDisplayPage(int tx, int ty, int tw, int th)
 
 void OptionsDialog::createMiscPage(int tx, int ty, int tw, int th)
 {
-  Fl_Group *group = new Fl_Group(tx, ty, tw, th, _("Misc."));
+  Fl_Group *group = new Fl_Group(tx, ty, tw, th, _("Miscellaneous"));
 
   tx += OUTER_MARGIN;
   ty += OUTER_MARGIN;
@@ -1025,7 +1049,7 @@ void OptionsDialog::createMiscPage(int tx, int ty, int tw, int th)
 }
 
 
-void OptionsDialog::handleAutoselect(Fl_Widget *widget, void *data)
+void OptionsDialog::handleAutoselect(Fl_Widget* /*widget*/, void *data)
 {
   OptionsDialog *dialog = (OptionsDialog*)data;
 
@@ -1042,7 +1066,7 @@ void OptionsDialog::handleAutoselect(Fl_Widget *widget, void *data)
 }
 
 
-void OptionsDialog::handleCompression(Fl_Widget *widget, void *data)
+void OptionsDialog::handleCompression(Fl_Widget* /*widget*/, void *data)
 {
   OptionsDialog *dialog = (OptionsDialog*)data;
 
@@ -1053,7 +1077,7 @@ void OptionsDialog::handleCompression(Fl_Widget *widget, void *data)
 }
 
 
-void OptionsDialog::handleJpeg(Fl_Widget *widget, void *data)
+void OptionsDialog::handleJpeg(Fl_Widget* /*widget*/, void *data)
 {
   OptionsDialog *dialog = (OptionsDialog*)data;
 
@@ -1065,7 +1089,7 @@ void OptionsDialog::handleJpeg(Fl_Widget *widget, void *data)
 }
 
 
-void OptionsDialog::handleX509(Fl_Widget *widget, void *data)
+void OptionsDialog::handleX509(Fl_Widget* /*widget*/, void *data)
 {
   OptionsDialog *dialog = (OptionsDialog*)data;
 
@@ -1079,7 +1103,7 @@ void OptionsDialog::handleX509(Fl_Widget *widget, void *data)
 }
 
 
-void OptionsDialog::handleRSAAES(Fl_Widget *widget, void *data)
+void OptionsDialog::handleRSAAES(Fl_Widget* /*widget*/, void *data)
 {
   OptionsDialog *dialog = (OptionsDialog*)data;
 
@@ -1090,8 +1114,9 @@ void OptionsDialog::handleRSAAES(Fl_Widget *widget, void *data)
 }
 
 
-void OptionsDialog::handleClipboard(Fl_Widget *widget, void *data)
+void OptionsDialog::handleClipboard(Fl_Widget* /*widget*/, void *data)
 {
+  (void)data;
 #if !defined(WIN32) && !defined(__APPLE__)
   OptionsDialog *dialog = (OptionsDialog*)data;
 
@@ -1106,7 +1131,7 @@ void OptionsDialog::handleClipboard(Fl_Widget *widget, void *data)
 #endif
 }
 
-void OptionsDialog::handleFullScreenMode(Fl_Widget *widget, void *data)
+void OptionsDialog::handleFullScreenMode(Fl_Widget* /*widget*/, void *data)
 {
   OptionsDialog *dialog = (OptionsDialog*)data;
 
@@ -1117,7 +1142,7 @@ void OptionsDialog::handleFullScreenMode(Fl_Widget *widget, void *data)
   }
 }
 
-void OptionsDialog::handleCancel(Fl_Widget *widget, void *data)
+void OptionsDialog::handleCancel(Fl_Widget* /*widget*/, void *data)
 {
   OptionsDialog *dialog = (OptionsDialog*)data;
 
@@ -1125,7 +1150,7 @@ void OptionsDialog::handleCancel(Fl_Widget *widget, void *data)
 }
 
 
-void OptionsDialog::handleOK(Fl_Widget *widget, void *data)
+void OptionsDialog::handleOK(Fl_Widget* /*widget*/, void *data)
 {
   OptionsDialog *dialog = (OptionsDialog*)data;
 
@@ -1156,5 +1181,5 @@ void OptionsDialog::handleScreenConfigTimeout(void *data)
 
     assert(self);
 
-    self->monitorArrangement->set(fullScreenSelectedMonitors.getParam());
+    self->monitorArrangement->value(fullScreenSelectedMonitors.getParam());
 }
