@@ -30,7 +30,7 @@ class Fl_Round_Button;
 class Fl_Input;
 class Fl_Int_Input;
 class Fl_Choice;
-class MonitorArrangement;
+class Fl_Monitor_Arrangement;
 
 typedef void (OptionsCallback)(void*);
 
@@ -135,7 +135,7 @@ protected:
   Fl_Round_Button *currentMonitorButton;
   Fl_Round_Button *allMonitorsButton;
   Fl_Round_Button *selectedMonitorsButton;
-  MonitorArrangement *monitorArrangement;
+  Fl_Monitor_Arrangement *monitorArrangement;
 
   /* Misc. */
   Fl_Check_Button *sharedCheckbox;
