@@ -34,12 +34,12 @@
 #include <FL/Fl_Return_Button.H>
 #include <FL/Fl_Pixmap.H>
 
-#include <rfb/util.h>
-#include <rfb/Password.h>
 #include <rfb/Exception.h>
+#include <rfb/obfuscate.h>
 
+#include "fltk/layout.h"
+#include "fltk/util.h"
 #include "i18n.h"
-#include "fltk_layout.h"
 #include "parameters.h"
 #include "UserDialog.h"
 
@@ -70,25 +70,38 @@ UserDialog::~UserDialog()
 {
 }
 
-void UserDialog::getUserPasswd(bool secure, char** user, char** password)
+void UserDialog::getUserPasswd(bool secure, std::string* user,
+                               std::string* password)
 {
-  CharArray passwordFileStr(passwordFile.getData());
+  const char *passwordFileName(passwordFile);
 
   assert(password);
+  char *envUsername = getenv("VNC_USERNAME");
+  char *envPassword = getenv("VNC_PASSWORD");
 
-  if (!user && passwordFileStr.buf[0]) {
-    ObfuscatedPasswd obfPwd(256);
+  if(user && envUsername && envPassword) {
+    *user = envUsername;
+    *password = envPassword;
+    return;
+  }
+
+  if (!user && envPassword) {
+    *password = envPassword;
+    return;
+  }
+
+  if (!user && passwordFileName[0]) {
+    std::vector<uint8_t> obfPwd(8);
     FILE* fp;
 
-    fp = fopen(passwordFileStr.buf, "rb");
+    fp = fopen(passwordFileName, "rb");
     if (!fp)
       throw rfb::Exception(_("Opening password file failed"));
 
-    obfPwd.length = fread(obfPwd.buf, 1, obfPwd.length, fp);
+    obfPwd.resize(fread(obfPwd.data(), 1, obfPwd.size(), fp));
     fclose(fp);
 
-    PlainPasswd passwd(obfPwd);
-    *password = passwd.takeBuf();
+    *password = deobfuscate(obfPwd.data(), obfPwd.size());
 
     return;
   }
@@ -100,9 +113,9 @@ void UserDialog::getUserPasswd(bool secure, char** user, char** password)
   Fl_Box *icon;
   Fl_Button *button;
 
-  int y;
+  int x, y;
 
-  win = new Fl_Window(410, 145, _("VNC authentication"));
+  win = new Fl_Window(410, 0, _("VNC authentication"));
   win->callback(button_cb,(void *)0);
 
   banner = new Fl_Box(0, 0, win->w(), 20);
@@ -118,23 +131,26 @@ void UserDialog::getUserPasswd(bool secure, char** user, char** password)
     banner->image(insecure_icon);
   }
 
-  y = 20 + 10;
+  x = OUTER_MARGIN;
+  y = banner->h() + OUTER_MARGIN;
 
-  icon = new Fl_Box(10, y, 50, 50, "?");
+  /* Mimic a fl_ask() box */
+  icon = new Fl_Box(x, y, 50, 50, "?");
   icon->box(FL_UP_BOX);
   icon->labelfont(FL_TIMES_BOLD);
   icon->labelsize(34);
   icon->color(FL_WHITE);
   icon->labelcolor(FL_BLUE);
 
-  y += 5;
+  x += icon->w() + INNER_MARGIN;
+  y += INNER_MARGIN;
 
   if (user) {
-    (new Fl_Box(70, y, win->w()-70-10, 20, _("Username:")))
-      ->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
-    y += 20 + 5;
-    username = new Fl_Input(70, y, win->w()-70-10, 25);
-    y += 25 + 5;
+    y += INPUT_LABEL_OFFSET;
+    username = new Fl_Input(x, y, win->w()- x - OUTER_MARGIN,
+                            INPUT_HEIGHT, _("Username:"));
+    username->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
+    y += INPUT_HEIGHT + INNER_MARGIN;
   } else {
     /*
      * Compiler is not bright enough to understand that
@@ -143,24 +159,30 @@ void UserDialog::getUserPasswd(bool secure, char** user, char** password)
     username = NULL;
   }
 
-  (new Fl_Box(70, y, win->w()-70-10, 20, _("Password:")))
-    ->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
-  y += 20 + 5;
-  passwd = new Fl_Secret_Input(70, y, win->w()-70-10, 25);
-  y += 25 + 5;
+  y += INPUT_LABEL_OFFSET;
+  passwd = new Fl_Secret_Input(x, y, win->w()- x - OUTER_MARGIN,
+                               INPUT_HEIGHT, _("Password:"));
+  passwd->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
+  y += INPUT_HEIGHT + INNER_MARGIN;
 
-  y += 5;
+  x = win->w() - OUTER_MARGIN;
+  y += OUTER_MARGIN - INNER_MARGIN;
 
-  button = new Fl_Return_Button(310, y, 90, 25, fl_ok);
-  button->align(FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
+  x -= BUTTON_WIDTH;
+  button = new Fl_Return_Button(x, y, BUTTON_WIDTH,
+                                BUTTON_HEIGHT, fl_ok);
   button->callback(button_cb, (void*)0);
+  x -= INNER_MARGIN;
 
-  button = new Fl_Button(210, y, 90, 25, fl_cancel);
-  button->align(FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
+  x -= BUTTON_WIDTH;
+  button = new Fl_Button(x, y, BUTTON_WIDTH, BUTTON_HEIGHT, fl_cancel);
   button->callback(button_cb, (void*)1);
   button->shortcut(FL_Escape);
+  x -= INNER_MARGIN;
 
-  y += 25 + 10;
+  y += BUTTON_HEIGHT;
+
+  y += OUTER_MARGIN;
 
   win->end();
 
@@ -175,8 +197,8 @@ void UserDialog::getUserPasswd(bool secure, char** user, char** password)
 
   if (ret_val == 0) {
     if (user)
-      *user = strDup(username->value());
-    *password = strDup(passwd->value());
+      *user = username->value();
+    *password = passwd->value();
   }
 
   delete win;
