@@ -28,7 +28,6 @@
 
 #ifdef _WIN32
 #include <windows.h>
-#include <tchar.h>
 #endif
 
 #include "parameters.h"
@@ -428,7 +427,7 @@ void saveHistoryToRegKey(const vector<string>& serverHistory) {
   if (res != ERROR_SUCCESS)
     throw rdr::SystemException(_("Failed to create registry key"), res);
 
-  size_t index = 0;
+  unsigned index = 0;
   assert(SERVER_HISTORY_SIZE < 100);
   char indexString[3];
 
@@ -518,7 +517,7 @@ void loadHistoryFromRegKey(vector<string>& serverHistory) {
     throw rdr::SystemException(_("Failed to open registry key"), res);
   }
 
-  size_t index;
+  unsigned index;
   const DWORD buffersize = 256;
   char indexString[3];
 
@@ -630,12 +629,11 @@ void saveViewerParameters(const char *filename, const char *servername) {
     return;
 #endif
     
-    char* homeDir = NULL;
-    if (getvnchomedir(&homeDir) == -1)
-      throw Exception(_("Could not obtain the home directory path"));
+    const char* configDir = os::getvncconfigdir();
+    if (configDir == NULL)
+      throw Exception(_("Could not obtain the config directory path"));
 
-    snprintf(filepath, sizeof(filepath), "%sdefault.tigervnc", homeDir);
-    delete[] homeDir;
+    snprintf(filepath, sizeof(filepath), "%s/default.tigervnc", configDir);
   } else {
     snprintf(filepath, sizeof(filepath), "%s", filename);
   }
@@ -682,7 +680,7 @@ void saveViewerParameters(const char *filename, const char *servername) {
 
 static bool findAndSetViewerParameterFromValue(
   VoidParameter* parameters[], size_t parameters_len,
-  char* value, char* line, char* filepath)
+  char* value, char* line)
 {
   const size_t buffersize = 256;
   char decodingBuffer[buffersize];
@@ -735,12 +733,11 @@ char* loadViewerParameters(const char *filename) {
     return loadFromReg();
 #endif
 
-    char* homeDir = NULL;
-    if (getvnchomedir(&homeDir) == -1)
-      throw Exception(_("Could not obtain the home directory path"));
+    const char* configDir = os::getvncconfigdir();
+    if (configDir == NULL)
+      throw Exception(_("Could not obtain the config directory path"));
 
-    snprintf(filepath, sizeof(filepath), "%sdefault.tigervnc", homeDir);
-    delete[] homeDir;
+    snprintf(filepath, sizeof(filepath), "%s/default.tigervnc", configDir);
   } else {
     snprintf(filepath, sizeof(filepath), "%s", filename);
   }
@@ -821,11 +818,11 @@ char* loadViewerParameters(const char *filename) {
 
       } else {
         invalidParameterName = findAndSetViewerParameterFromValue(parameterArray, sizeof(parameterArray),
-                                                                  value, line, filepath);
+                                                                  value, line);
 
         if (invalidParameterName) {
           invalidParameterName = findAndSetViewerParameterFromValue(readOnlyParameterArray, sizeof(readOnlyParameterArray),
-                                                                    value, line, filepath);
+                                                                    value, line);
         }
       }
     } catch(Exception& e) {
