@@ -19,10 +19,21 @@
 #ifndef __VNCVIEWER_H__
 #define __VNCVIEWER_H__
 
-#define VNCSERVERNAMELEN 64
+#define VNCSERVERNAMELEN 256
+
+namespace rdr {
+  struct Exception;
+};
+
+void abort_vncviewer(const char *error, ...)
+  __attribute__((__format__ (__printf__, 1, 2)));
+void abort_connection(const char *error, ...)
+  __attribute__((__format__ (__printf__, 1, 2)));
+void abort_connection_with_unexpected_error(const rdr::Exception &);
+
+void disconnect();
+bool should_disconnect();
 
-void exit_vncviewer(const char *error = NULL);
-bool should_exit();
 void about_vncviewer();
 
 #endif
